reasoning_models = (
    "o3-mini-2025-01-31",
    "o3-mini",
    "o1",
    "o1-2024-12-17",
)

reasoning_not_support_params = (
    "temperature",
    "top_p",
    "logit_bias",
    "logprobs",
    "top_logprobs",
)
